<?php
class VLCServer {
	/**
	* @access private
	* @var int
	*/
	private $lVolume;

	/**
	* @access private
	* @var int
	*/
	private $lScale;

	/**
	* @access private
	* @var string
	*/
	private $lVLCExecutable;

	/**
	* @access private
	* @var int
	*/
	private $lChannels;
	
	
	/**
	* @access private
	* @var array
	*/
	private $lProfiles;
	
	private $lSuccess;

	/**
	* Constructor. This creates a VLC Server object. Some defaults are already set. These are: volume, scale and audio channels. It alse detects if it is running on Windows or Linux.
	* @return VLCServer
	*/
	function __construct() {		
		$this->lProfiles = array();
		
		/*
			Here you can change some codec settings.
			Possible Audio codecs:
				1. mp4a
				2. mp3
				3. mpga
		*/
		
		//3G Profile (BETA)
		//$this->lProfiles[] = array("name" => "3G", "width" => 240, "height" => 160, "videocodec" => "h264", "videobitrate" => 150, "videofps" => 15, "audiocodec" => "mp4a", "audiobitrate" => 32, "audiosamplerate" => 44100);
		// Mobile Profile
		$this->lProfiles[] = array("name" => "Mobile", "width" => 240, "height" => 160, "videocodec" => "h264", "videobitrate" => 150, "videofps" => 25, "audiocodec" => "mp4a", "audiobitrate" => 64, "audiosamplerate" => 48000);
		//Main Profile
		$this->lProfiles[] = array("name" => "SD", "width" => 720, "height" => 404, "videocodec" => "h264", "videobitrate" => 512, "videofps" => 25, "audiocodec" => "mp4a", "audiobitrate" => 92, "audiosamplerate" => 48000);
		//HD Profile
		//$this->lProfiles[] = array("name" => "HD", "width" => 1280, "height" => 720, "videocodec" => "h264", "videobitrate" => 2048, "videofps" => 25, "audiocodec" => "mp4a", "audiobitrate" => 192, "audiosamplerate" => 48000);
		
		$this->lVolume = 80;
		$this->lScale = 1;
		$this->lChannels = 2;
		$this->lSuccess = false;
		
		$this->lVLCExecutable = Settings::getVLCExecutable();	
	}

	/**
	* Start the VLC Server by executing the vlc command line. You have to set all options before starting this server. If the server is started, you should be able to connect to it and watch some tv.
	* @param boolean $pDebug
	* @return string
	*/
	public function startServer($pStreamURL,$pChannelObj) {
	//	return 1;
		global $gDebugObj;
		$lCurrentStatus = $this->getCurrentStatus();
		if ($lCurrentStatus["streamurl"] != $pStreamURL) {
			$this->stopServer();
			sleep(2);
			if (Settings::getGgrabLocation() != "") {
				$lCMD = Settings::getGgrabLocation() . " -host " . Settings::getDreamboxIP() . " -q -o - -p 0x1006 0x1106 | " . $this->lVLCExecutable . " - ";
			} else {
				$lCMD = $this->lVLCExecutable . " -I dummy \"" . $pStreamURL . "\" vlc://quit --audio-language=" . Settings::getVLCAudioLanguage() . " --sub-language=" . Settings::getVLCSubtitleLanguage();
			}
			$lPortNumbers = 2000;
			$lTransCode = "#duplicate{";
			foreach ($this->lProfiles as $lProfile) {
				switch ($lProfile["videocodec"]) {
					case "h264":
						$lExtraEncoderSettings = "venc=x264{aud,keyint=25,profile=baseline,level=30,bframes=0,ref=1,nocabac,qpmax=36,qpmin=10,me=hex,subme=9,qcomp=0.6,fps=" . ($lProfile["videofps"]) . "},";
					break;
					default:
						$lExtraEncoderSettings = "";
					break;
				}
				if (Settings::isVLCiPhoneEnabled()) {
					$lTransCode .= "dst={deinterlace,transcode{vcodec=". $lProfile["videocodec"] . "," . $lExtraEncoderSettings . "vb=". $lProfile["videobitrate"] . ",scale=". $this->lScale .",acodec=". $lProfile["audiocodec"] .",ab=" . $lProfile["audiobitrate"] . ",channels=". $this->lChannels .",samplerate=" . $lProfile["audiosamplerate"] . ",width=". $lProfile["width"] .",height=". $lProfile["height"] . ",audio-sync,fps=" . ($lProfile["videofps"]) . "}";
					$lTransCode .= ":gather:std{access=livehttp{seglen=10,delsegs=true,numsegs=5,index=" . getcwd() . "/iphone/stream/" . $pChannelObj->getName(true) . "_" . $lProfile["name"] . ".m3u8,index-url=" . $pChannelObj->getName(true) . "_" . $lProfile["name"] . "-########.ts},mux=ts{use-key-frames},dst=" . getcwd() . "/iphone/stream/" . $pChannelObj->getName(true) . "_" . $lProfile["name"] . "-########.ts}},";
				}
				$lTransCode .= "dst={transcode{deinterlace,vcodec=". $lProfile["videocodec"] . "," . $lExtraEncoderSettings . "vb=". $lProfile["videobitrate"] . ",scale=". $this->lScale .",acodec=". $lProfile["audiocodec"] .",ab=" . $lProfile["audiobitrate"] . ",channels=". $this->lChannels .",samplerate=" . $lProfile["audiosamplerate"] . ",width=". $lProfile["width"] .",height=". $lProfile["height"] . ",audio-sync,fps=" . ($lProfile["videofps"]) . "}";
				if (Settings::isWowzaEnabled()) {
					$lTransCode .= ":rtp{dst=". Settings::getVLCIP() .",port=" . $lPortNumbers . ",mux=ts,ttl=10,sdp=http://". Settings::getVLCIP() .":". Settings::getVLCPort() ."/" . $this->vlcStreamUrlFormat($pChannelObj->getName() . "_" . $lProfile["name"]) . ".sdp}";
				} else {
					$lTransCode .= ":gather:std{access=http,mime=\"video/mp4\",dst=" . Settings::getVLCIP() .":". Settings::getVLCPort() . "/" . $this->vlcStreamUrlFormat($pChannelObj->getName() . "_" . $lProfile["name"]) . ".mp4,mux=ffmpeg{mux=flv}}";
				}
				$lPortNumbers += 10;
				//if (Settings::isVLCiPhoneEnabled() && $lProfile["name"] == 'Mobile') $lTransCode .= "}";
				$lTransCode .= "},";
			}
			$lTransCode = substr($lTransCode,0,-1) . "}";
			if (Utils::isLinux() || Utils::isMacOSX()) $lTransCode = "\"" . $lTransCode . "\"";
			$lCMD .= " --sout=" . $lTransCode;
			Utils::execute($lCMD);
			//if (Utils::isLinux()) $this->startChunker($pChannelObj->getName());
			if (Settings::isVLCiPhoneEnabled()) $this->setIphoneStream($pChannelObj);
			if (Settings::isWowzaEnabled()) $this->saveSDPfiles($pChannelObj);
			if (Settings::getDebug()) $gDebugObj->setDebugMessage("VLC Command:\n".$lCMD);
		}
		return $this->getPlayerUrl($pChannelObj);
	}
	
	public function getPlayerUrl($pChannelObj) {
		if (Utils::isiPhone()) {
			if (Settings::isFullWowzaServer()) {
				if (Settings::isMultiBitRate()) {
					$lReturnValue = "http://" . Settings::getWowzaServerName() . ":1935/dreambox/" . $this->vlcStreamUrlFormat($pChannelObj->getName()) . ".smil/playlist.m3u8";
				} else {
					$lReturnValue = "http://" . Settings::getWowzaServerName() . ":1935/dreambox/" . $this->vlcStreamUrlFormat($pChannelObj->getName()) . "_3G.sdp/playlist.m3u8";
				}
			} else if (Settings::isVLCiPhoneEnabled()) {
				$lReturnValue = $pChannelObj->getName(true);
			} else {
				$lReturnValue = '';
			}
		} else if (Utils::isMobileDevice()) {
			$lReturnValue = "rtsp://" . gethostbyname(Settings::getWowzaServerName()) . ":1935/dreambox/" . $this->vlcStreamUrlFormat($pChannelObj->getName()) . "_Mobile.sdp";
		} else if (Settings::isWowzaEnabled()) {
			$lReturnValue = "{streamer:'rtmp://" . Settings::getWowzaServerName() . "/dreambox',levels: [";
			foreach ($this->lProfiles as $lProfile) {
				if ($lProfile["name"] == "3G") continue;
				$lReturnValue .= "{bitrate: " . ($lProfile["videobitrate"]+$lProfile["audiobitrate"]) . ", file: '" . $this->vlcStreamUrlFormat($pChannelObj->getName() . "_" . $lProfile["name"]) . ".sdp', width: " . $lProfile["width"] . " },";
			}
			$lReturnValue = substr($lReturnValue,0,-1) .  "],type:'rtmp'}";
		} else {
			$lReturnValue = "{levels: [";
			foreach ($this->lProfiles as $lProfile) {
				if ($lProfile["name"] == "3G") continue;
				$lReturnValue .= "{bitrate: " . ($lProfile["videobitrate"]+$lProfile["audiobitrate"]) . ", file: 'http://" . Settings::getVLCPlayerIP() . ":" . Settings::getVLCPort() . "/" . $this->vlcStreamUrlFormat($pChannelObj->getName() . "_" . $lProfile["name"]) . ".mp4', width: " . $lProfile["width"] . " },";
			}
			$lReturnValue = substr($lReturnValue,0,-1) .  "],type:'video'}";
		}
		return $lReturnValue;
	}

	public function getCurrentStatus() {
		$lReturnArray = array();
		$lReturnArray["channel"] = "";
		$lReturnArray["streamurl"] = "";
		if (Utils::isLinux() || Utils::isMacOSX()) {
			$lData = trim(shell_exec("ps ax | grep -i vlc | grep -v grep"));
			$lStartPos = stripos($lData,$this->lVLCExecutable);
			$lData = trim(substr($lData,strlen($this->lVLCExecutable . " -I dummy")+$lStartPos));
			$lReturnArray["streamurl"] = trim(substr($lData,0,stripos($lData,"vlc://quit")));
			if (substr($lReturnArray["streamurl"],0,7) == "http://") {
				$lReturnArray["channel"] = explode("/",$lReturnArray["streamurl"]);
				$lReturnArray["channel"] = end($lReturnArray["channel"]);
			} else {
				$lReturnArray["channel"] = $lReturnArray["streamurl"];
			}
		}
		$lReturnArray["channel"] = str_replace("%20"," ",$lReturnArray["channel"]);
		return $lReturnArray;
	}

	private function startChunker($pChannelName) {
		$lChunkerFolder = "apps/chunker/";
		$pChannelName = str_replace(" ","_",$pChannelName);
		$lCurrentDir = getcwd();
		chdir($lChunkerFolder);
		shell_exec("rm ../../iphone/stream/*");
		shell_exec("rm tmp/*.ts");
		$lChunkerConfigFileContents = file_get_contents("dreambox.temp.yml");
		$lChunkerConfigFileContents = str_replace("[CHANNEL]",$pChannelName,$lChunkerConfigFileContents);
		if (Settings::isWowzaEnabled()) {
			$lChunkerConfigFileContents = str_replace("[FULL_RTSP_URL]","rtsp://" . Settings::getWowzaServerName() . "/dreambox/" . $pChannelName . "_Mobile.sdp",$lChunkerConfigFileContents);
		} else {
			$lChunkerConfigFileContents = str_replace("[FULL_RTSP_URL]","http://" . Settings::getVLCIP() . ":" . Settings::getVLCPort() . "/restream.mp4?" . $pChannelName,$lChunkerConfigFileContents);
		}
		file_put_contents("./dreambox.yml",$lChunkerConfigFileContents);
		shell_exec("/usr/bin/killall -Iq ruby");
		shell_exec("/usr/bin/killall -Iq ffmpeg");
		sleep(3);
		shell_exec("./http_streamer.rb dreambox.yml >/dev/null &");
		chdir($lCurrentDir);
	}

	/**
	* Stop the VLC Server. This is handy to spare some system resources.
	*/
	public function stopServer() {
		if (Utils::isLinux()) {
			$lCMD = "rm " . getcwd() . "/apps/wowza/content/*.s*;killall -I -q vlc ruby ffmpeg";
		} elseif(Utils::isMacOSX()) {
			$lCMD = "rm " . getcwd() . "/apps/wowza/content/*.s*;killall VLC";
		} elseif(Utils::isWindows()) {
			// But you need to download this one: http://www.beyondlogic.org/solutions/processutil/processutil.htm
			// And install it in a dir called apps in the restream root dir
			$lCMD = "del " . getcwd() . "/apps/wowza/content/*.s*";
			Utils::execute($lCMD);
			$lCMD = getcwd() . "\apps\Process.exe -k vlc.exe";
		} else {
			$lCMD = "";
		}
		Utils::execute($lCMD);
	}

	private function getProgramIDs() {
		if (Utils::isLinux()) {

		}
	}
	
	private function saveSDPfiles($pChannelObj) {
		$this->lSuccess = false;
		$lSmillContent = "<smil><head></head><body><switch>";
		foreach($this->lProfiles as $lProfile) {
			
			$lVLCLocation = "http://". Settings::getVLCIP() .":". Settings::getVLCPort();
			$lSDPFileName = $this->vlcStreamUrlFormat($pChannelObj->getName() . "_" . $lProfile["name"]) . ".sdp";
			$lSDPContent = "";
		
			for ($i = 0; $i < 15; $i++) {
				$lSDPContent = @file_get_contents($lVLCLocation ."/" . $lSDPFileName);
				if ($lSDPContent != "" && ((stripos($lSDPContent,"m=video") !== false ) || ($i > 12 && stripos($lSDPContent,"m=audio") !== false))) {
					file_put_contents(getcwd() . "/apps/wowza/content/" . $lSDPFileName,$lSDPContent);
					if ($lProfile["name"] != "3G") $lSmillContent .= "<video src=\"" . $lSDPFileName . "\" system-bitrate=\"" . (($lProfile["videobitrate"] + $lProfile["audiobitrate"]) * 1024). "\"/>";
					$this->lSuccess = true;
					break;
				} 
				sleep(1);
			}
		}
		$lSmillContent .= "</switch></body></smil>";
		if (Settings::isMultiBitRate()) file_put_contents(getcwd() . "/apps/wowza/content/" . $this->vlcStreamUrlFormat($pChannelObj->getName()) . ".smil",$lSmillContent);
	}
	
	private function setIphoneStream($pChannelObj) {
		exec("rm " . getcwd() . "/iphone/stream/*");
		$lsave = false;
		$lMultiBitrate = "#EXTM3U\n";
		foreach ($this->lProfiles as $lProfile) {
			for ($i = 0; $i < 15; $i++) {
				if (file_exists(getcwd() . "/iphone/stream/" . $pChannelObj->getName(true) . "_" . $lProfile["name"] . ".m3u8")) {
					$lMultiBitrate .= "#EXT-X-STREAM-INF:PROGRAM-ID=1, BANDWIDTH=" . round(($lProfile["videobitrate"]+$lProfile["audiobitrate"]) * 1024) . "\n" . $pChannelObj->getName(true) . "_" . $lProfile["name"] . ".m3u8\n";
					$lsave = true;
					break;
				}
				sleep(1);
			}			
		}
		
		if ($lsave) file_put_contents(getcwd() . "/iphone/stream/" . $pChannelObj->getName(true) . ".m3u8",$lMultiBitrate);
	}

	private function vlcStreamUrlFormat($pUrl) {
		$pUrl = trim($pUrl);
		return str_replace(" ","_",preg_replace("/[^a-zA-Z0-9_\s]/","",$pUrl));
	}

	/**
	* Check if the server is still running.
	* @return boolean
	*/
	public function isServerRunning() {
		if (Utils::isLinux() || Utils::isMacOSX()) {
			$lData = trim(shell_exec("ps ax | grep -i vlc | grep -v grep | grep " . Settings::getDreamboxIP()));
		} elseif(Utils::isWindows()) {
			// On windows it is not possible to distinquish different vlc processes... so if there is a VLC proces, the server is running...
			$lCMD = getcwd() . "\apps\Process.exe | " . getcwd()  . "\apps\grep.exe vlc";
			$lData = trim(shell_exec($lCMD));
		}
		return (strlen($lData) > 0);
	}
}
?>
